(require-library simple-tests simple-exceptions)
(import simple-tests simple-exceptions)

(define-test (simple-exceptions?)
  (check
    (define foo-exn (make-exception "foo-msg"))
    (define bar-exn (make-exception "bar-msg" 'bar))
    (exception? (foo-exn 'nowhere))

    (bar-exn 'nowhere)
    (exception? (bar-exn 'nowhere))

    ((exception-of? 'bar) (bar-exn 'nowhere))
    (not ((exception-of? 'bar) (foo-exn 'nowhere)))
    
    (equal?
      (arguments ((make-exception "msg" 'baz) 'nowhere "bar"))
      (list "bar"))

    ((exception-of? 'key)
     ((make-exception "msg" 'key) 'nowhere))

    (define list-empty-exn
      (make-exception "argument list empty" 'list-empty))

    (define (try-car lst)
      (if (null? lst)
        (raise (list-empty-exn 'try-car lst))
        (car lst)))
    
    ;; exception handler procedure
    (not (with-handler
           (lambda (exn)
             (if ((exception-of? 'list-empty) exn)
               #f
               #t))
           (lambda () (try-car '()))))
    
    (zero? (with-handler (lambda (e) 0) (lambda () (/ 5 0))))

    ;; the three high-level exception handler macros
    (not (condition-case (try-car '())
           ((exn list-empty) #f)))

    (null? (guard
             (exn (((exception-of? 'list-empty) exn)
                   (car (arguments exn)))
                  (else #f))
             (try-car '())))

    (null? (handle-exceptions exn
             (if ((exception-of? 'list-empty) exn)
               (car (arguments exn))
               #f)
             (try-car '())))

    (= (guard
         (exn ((assq 'a exn) => cdr)
              ((assq 'b exn)))
         (raise (list (cons 'a 42))))
       42)
    (equal? (guard
              (exn ((assq 'a exn) => cdr)
                   ((assq 'b exn)))
              (raise (list (cons 'b 23))))
            '(b . 23))
    (eq? (guard
           (exn (((exception-of? 'foo) exn)
                 (location exn))
                ((exception? exn)
                 (message exn))
                (else
                  (arguments exn)))
           (raise ((make-exception "msg" 'foo)
                    'location-unknown)))
          'location-unknown)
    (string=? (guard
                (exn (((exception-of? 'foo) exn)
                      (location exn))
                     ((exception? exn)
                      (message exn))
                     (else
                       (arguments exn)))
                (raise ((make-exception "nothing" 'bar) 'loc)))
              "nothing")
    (not (guard (exn (((exception-of? 'foo) exn)
                      (location exn))
                     ((exception? exn)
                      (message exn))
                     (else
                       #f)) 
                (raise 'bar)))

    (not (condition-case (assert* 'nowhere
                                  (= 1 1)
                                  (= 1 2))
           ((exn assert) #f)))
    ))

(compound-test (SIMPLE-EXEPTIONS)
  (simple-exceptions?)
  )


